@@ -150,13 +150,8 @@ update_other_user_visibility (GdmUserChooserWidget *widget)
                 goto out;
         }
 
-        number_of_users = gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (widget));
-
-        /* we hide the Other user if it's the last one, and we show it
-         * if there's another user */
-        if (number_of_users == 1 && widget->priv->has_user_other) {
-                remove_user_other (widget);
-        } if (number_of_users >= 1 && !widget->priv->has_user_other) {
+        /* Always show the Other user if requested */
+        if (!widget->priv->has_user_other) {
                 add_user_other (widget);
         }
 
@@ -185,6 +180,9 @@ update_item_for_user (GdmUserChooserWidget *widget,
         char         *escaped_username;
         char         *escaped_real_name;
 
+        if (!gdm_user_is_loaded (user)) {
+                return;
+        }
 
         size = get_icon_height_for_widget (GTK_WIDGET (widget));
         pixbuf = gdm_user_render_icon (user, size);
@@ -235,6 +233,14 @@ on_item_load (GdmChooserWidget     *widget,
                 return;
         }
 
+        if (strcmp (id, GDM_USER_CHOOSER_USER_OTHER) == 0) {
+                return;
+        }
+
+        if (strcmp (id, GDM_USER_CHOOSER_USER_GUEST) == 0) {
+                return;
+        }
+
         user = gdm_user_manager_get_user (user_chooser->priv->manager, id);
         if (user != NULL) {
                 update_item_for_user (user_chooser, user);
@@ -473,7 +479,11 @@ add_user (GdmUserChooserWidget *widget,
                 return;
         }
 
-        pixbuf = g_object_ref (widget->priv->stock_person_pixbuf);
+        if (widget->priv->stock_person_pixbuf != NULL) {
+                pixbuf = g_object_ref (widget->priv->stock_person_pixbuf);
+        } else {
+                pixbuf = NULL;
+        }
 
         tooltip = g_strdup_printf (_("Log in as %s"),
                                    gdm_user_get_user_name (user));
@@ -823,6 +833,9 @@ get_pixbuf_from_icon_names (GdmUserChooserWidget *widget,
         int          size;
         const char  *icon_name;
         va_list      argument_list;
+        gint        *sizes;
+        gint         candidate_size;
+        int          i;
 
         array = g_ptr_array_new ();
 
@@ -839,9 +852,29 @@ get_pixbuf_from_icon_names (GdmUserChooserWidget *widget,
 
         size = get_icon_height_for_widget (GTK_WIDGET (widget));
 
+        sizes = gtk_icon_theme_get_icon_sizes (widget->priv->icon_theme, first_name);
+
+        candidate_size = 0;
+        for (i = 0; sizes[i] != 0; i++) {
+
+                /* scalable */
+                if (sizes[i] == -1) {
+                        candidate_size = sizes[i];
+                        break;
+                }
+
+                if (ABS (size - sizes[i]) < ABS (size - candidate_size)) {
+                        candidate_size = sizes[i];
+                }
+        }
+
+        if (candidate_size == 0) {
+                candidate_size = size;
+        }
+
         icon_info = gtk_icon_theme_choose_icon (widget->priv->icon_theme,
                                                 (const char **) array->pdata,
-                                                size,
+                                                candidate_size,
                                                 GTK_ICON_LOOKUP_GENERIC_FALLBACK);
         g_ptr_array_free (array, FALSE);
 
@@ -858,8 +891,16 @@ get_pixbuf_from_icon_names (GdmUserChooserWidget *widget,
                         g_error_free (error);
                 }
         } else {
+                GdkPixbuf *scaled_pixbuf;
+
+                guchar pixel = 0x00000000;
+
                 g_warning ("Could not find icon '%s' or fallbacks", first_name);
-                pixbuf = NULL;
+                pixbuf = gdk_pixbuf_new_from_data (&pixel, GDK_COLORSPACE_RGB,
+                                                   TRUE, 8, 1, 1, 1, NULL, NULL);
+                scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf, size, size, GDK_INTERP_NEAREST);
+                g_object_unref (pixbuf);
+                scaled_pixbuf = pixbuf;
         }
 
         return pixbuf;
@@ -908,7 +949,14 @@ update_icons (GdmChooserWidget *widget,
               IconUpdateData   *data)
 {
         if (data->old_icon == *image) {
+                if (*image != NULL) {
+                        g_object_unref (*image);
+                }
                 *image = data->new_icon;
+
+                if (*image != NULL) {
+                        g_object_ref (*image);
+                }
                 return TRUE;
         }
 
